package dev.shadowsoffire.apotheosis.adventure.loot;

import java.util.stream.Collector;
import java.util.stream.Collectors;
import dev.shadowsoffire.apotheosis.adventure.compat.GameStagesCompat;
import net.minecraft.world.level.Level;
import dev.shadowsoffire.placebo.reload.WeightedDynamicRegistry;
import java.util.function.Predicate;
import net.minecraft.world.level.ServerLevelAccessor;
import net.minecraft.world.entity.player.Player;
import javax.annotation.Nullable;
import net.minecraft.network.chat.MutableComponent;
import java.util.Iterator;
import java.util.Set;
import java.util.Map;
import net.minecraft.network.chat.Style;
import net.minecraft.network.chat.Component;
import java.util.List;
import java.util.Collections;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixHelper;
import dev.shadowsoffire.apotheosis.Apoth;
import dev.shadowsoffire.placebo.codec.CodecProvider;
import java.util.ArrayList;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixInstance;
import java.util.HashMap;
import dev.shadowsoffire.apotheosis.adventure.socket.SocketHelper;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixType;
import org.apache.commons.lang3.mutable.MutableInt;
import dev.shadowsoffire.apotheosis.adventure.affix.Affix;
import dev.shadowsoffire.placebo.reload.DynamicHolder;
import java.util.LinkedHashSet;
import net.minecraft.util.RandomSource;
import net.minecraft.world.item.ItemStack;
import java.util.Random;

public class LootController
{
    static Random jRand;
    
    public static ItemStack createLootItem(final ItemStack stack, final LootRarity rarity, final RandomSource rand) {
        final LootCategory cat = LootCategory.forItem(stack);
        if (cat.isNone()) {
            return stack;
        }
        return createLootItem(stack, cat, rarity, rand);
    }
    
    public static ItemStack createLootItem(final ItemStack stack, final LootCategory cat, final LootRarity rarity, final RandomSource rand) {
        final Set<DynamicHolder<? extends Affix>> selected = new LinkedHashSet<DynamicHolder<? extends Affix>>();
        final MutableInt sockets = new MutableInt(0);
        float durability = 0.0f;
        for (final LootRarity.LootRule rule : rarity.getRules()) {
            if (rule.type() == AffixType.DURABILITY) {
                durability = rule.chance();
            }
            else {
                rule.execute(stack, rarity, selected, sockets, rand);
            }
        }
        sockets.setValue(Math.max(sockets.getValue(), SocketHelper.getSockets(stack)));
        final Map<DynamicHolder<? extends Affix>, AffixInstance> loaded = new HashMap<DynamicHolder<? extends Affix>, AffixInstance>();
        final List<AffixInstance> nameList = new ArrayList<AffixInstance>(selected.size());
        for (final DynamicHolder<? extends Affix> a : selected) {
            final AffixInstance inst = new AffixInstance((DynamicHolder)a, stack, RarityRegistry.INSTANCE.holder((CodecProvider)rarity), rand.m_188501_());
            loaded.put(a, inst);
            nameList.add(inst);
        }
        if (nameList.size() == 0) {
            throw new RuntimeException(String.format("Failed to locate any affixes for %s{%s} with category %s and rarity %s.", stack.m_41720_(), stack.m_41783_(), cat, rarity));
        }
        if (sockets.intValue() > 0) {
            SocketHelper.setSockets(stack, sockets.intValue());
        }
        if (durability > 0.0f) {
            loaded.put((DynamicHolder<? extends Affix>)Apoth.Affixes.DURABLE, new AffixInstance(Apoth.Affixes.DURABLE, stack, RarityRegistry.INSTANCE.holder((CodecProvider)rarity), durability + AffixHelper.step(-0.07f, 14, 0.01f).get(rand.m_188501_())));
        }
        LootController.jRand.setSeed(rand.m_188505_());
        Collections.shuffle(nameList, LootController.jRand);
        final String key = (nameList.size() > 1) ? "misc.apotheosis.affix_name.three" : "misc.apotheosis.affix_name.two";
        final MutableComponent name = Component.m_237110_(key, new Object[] { nameList.get(0).getName(true), "", (nameList.size() > 1) ? nameList.get(1).getName(false) : "" }).m_130948_(Style.f_131099_.m_131148_(rarity.getColor()));
        AffixHelper.setRarity(stack, rarity);
        AffixHelper.setAffixes(stack, (Map)loaded);
        AffixHelper.setName(stack, (Component)name);
        return stack;
    }
    
    public static ItemStack createRandomLootItem(final RandomSource rand, @Nullable LootRarity rarity, final Player player, final ServerLevelAccessor level) {
        final AffixLootEntry entry = (AffixLootEntry)AffixLootRegistry.INSTANCE.getRandomItem(rand, player.m_36336_(), new Predicate[] { WeightedDynamicRegistry.IDimensional.matches((Level)level.m_6018_()), GameStagesCompat.IStaged.matches(player) });
        if (entry == null) {
            return ItemStack.f_41583_;
        }
        if (rarity == null) {
            rarity = LootRarity.random(rand, player.m_36336_(), entry);
        }
        return createLootItem(entry.getStack(), entry.getType(), rarity, rand);
    }
    
    public static List<DynamicHolder<? extends Affix>> getAvailableAffixes(final ItemStack stack, final LootRarity rarity, final Set<DynamicHolder<? extends Affix>> currentAffixes, final AffixType type) {
        final LootCategory cat = LootCategory.forItem(stack);
        return AffixHelper.byType(type).stream().filter(a -> ((Affix)a.get()).canApplyTo(stack, cat, rarity)).filter(a -> !currentAffixes.contains(a)).collect((Collector<? super Object, ?, List<DynamicHolder<? extends Affix>>>)Collectors.toList());
    }
    
    static {
        LootController.jRand = new Random();
    }
}
